#!/usr/bin/env python3
# -*- coding=utf-8 -*-
# 本脚由亁颐堂现任明教教主编写，用于乾颐盾Python课程！
# 教主QQ:605658506
# 亁颐堂官网www.qytang.com
# 教主技术进化论拓展你的技术新边疆
# https://ke.qq.com/course/271956?tuin=24199d8a


from pysnmp.entity.rfc3413.oneliner import cmdgen


def snmpv2_getnext(ip, community, oid, port=161):
    cmd_gen = cmdgen.CommandGenerator()

    error_indication, error_status, error_index, var_bind_table = cmd_gen.nextCmd(
        cmdgen.CommunityData(community),  # 设置community
        cmdgen.UdpTransportTarget((ip, port)),  # 设置IP地址和端口号
        oid,  # 设置OID
    )
    # 错误处理
    if error_indication:
        print(error_indication)
    elif error_status:
        print(error_status)

    result = []
    # varBindTable是个list，元素的个数可能有好多个。它的元素也是list，这个list里的元素是ObjectType，个数只有1个。
    for var_bind_table_row in var_bind_table:
        for item in var_bind_table_row:
            result.append((item.prettyPrint().split("=")[0].strip(), item.prettyPrint().split("=")[1].strip()))
    return result


if __name__ == "__main__":
    # 使用Linux解释器 & WIN解释器
    # 显示接口信息
    print(snmpv2_getnext("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.2", port=161))
    for x, y in snmpv2_getnext("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.2", port=161):
        print(x, y)
    # 接口速率
    print(snmpv2_getnext("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.5", port=161))

    # 进接口字节数
    print(snmpv2_getnext("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.10", port=161))

    # 出接口字节数
    print(snmpv2_getnext("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.16", port=161))



#!/usr/bin/env python3
# -*- coding=utf-8 -*-
# 本脚由亁颐堂现任明教教主编写，用于乾颐盾Python课程！
# 教主QQ:605658506
# 亁颐堂官网www.qytang.com
# 教主技术进化论拓展你的技术新边疆
# https://ke.qq.com/course/271956?tuin=24199d8a
from pysnmp.entity.rfc3413.oneliner import cmdgen


def snmpv2_getbulk(ip, community, oid, count=25, port=161):
    cmd_gen = cmdgen.CommandGenerator()

    error_indication, error_status, error_index, var_bind_table = cmd_gen.bulkCmd(
        cmdgen.CommunityData(community),  # 配置community
        cmdgen.UdpTransportTarget((ip, port)),  # 配置IP地址和端口号
        0, count,  # 0为non-repeaters 和  25为max-repetitions(一个数据包中最多25个条目，和显示无关)
        oid,  # OID
    )

    """
    non-repeaters介绍
    the number of objects that are only expected to return a single GETNEXT instance, not multiple instances. Managers \
    frequently request the value of sysUpTime and only want that instance plus a list of other objects.
    max-repetitions介绍
    the number of objects that should be returned for all the repeating OIDs. Agent's must truncate the list to \
    something shorter if it won't fit within the max-message size supported by the command generator or the agent.
    详细介绍
    https://www.webnms.com/snmp/help/snmpapi/snmpv3/snmp_operations/snmp_getbulk.html
    """
    # 错误处理
    if error_indication:
        print(error_indication)
    elif error_status:
        print(error_status)

    result = []
    # varBindTable是个list，元素的个数可能有好多个。它的元素也是list，这个list里的元素是ObjectType，个数只有1个。
    for var_bind_table_row in var_bind_table:
        for item in var_bind_table_row:
            result.append((item.prettyPrint().split("=")[0].strip(), item.prettyPrint().split("=")[1].strip()))
    return result


if __name__ == "__main__":
    # 使用Linux解释器 & WIN解释器
    # 虽然count=25,但是脚本会自动过滤只显示主ID内的内容
    print(snmpv2_getbulk("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.2", count=25, port=161))
    for x, y in snmpv2_getbulk("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.2", count=25, port=161):
        print(x, y)
    # 接口速率
    print(snmpv2_getbulk("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.5", port=161))

    # 进接口字节数
    print(snmpv2_getbulk("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.10", port=161))

    # 出接口字节数
    print(snmpv2_getbulk("192.168.1.151", "QytangR0", "1.3.6.1.2.1.2.2.1.16", port=161))



#!/usr/bin/env python3
# _*_ coding=utf-8 _*_

# fulfill the functions of command on Cisco devices

import time
import paramiko


class QYTCiscoSSH:
    def __init__(self, hostname, username, password):
        self.client = paramiko.SSHClient()
        self.client.load_system_host_keys()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(hostname=hostname, username=username, password=password)

    def show_run(self):
        try:
            stdin, stdout, stderr = self.client.exec_command('show running-config')
            return stdout.read().decode()
        except Exception as e:
            print(e)
        finally:
            self.client.close()

    def show(self, show_cmds):
        return_result_list = []
        try:
            remote = self.client.invoke_shell()
            remote.send('term len 0\n')
            time.sleep(1)
            for cmd in show_cmds:
                remote.send('%s \n' % cmd)
                time.sleep(2)
                buf = remote.recv(65000)
                f = open('sshlogfile0001.txt', 'ab')
                f.write(buf)
                f.close()
                return_result_list.append(buf)
            return return_result_list
        except Exception as e:
            print(e)
        finally:
            self.client.close()

    def config(self, config_cmds):
        try:
            enable = 'Cisc0123'
            wait_time = 2
            verbose = True
            chan = self.client.invoke_shell()
            chan.send('term len 0\n')
            time.sleep(1)
            login_mode = chan.recv(2048).decode()
            if '>' in login_mode:
                chan.send("enable\n".encode())
                time.sleep(1)
                en_password = enable + '\n'
                chan.send(en_password)
            for cmd in config_cmds:
                chan.send(b'\n')
                chan.send(cmd.encode())
                chan.send(b'\n')
                x = chan.recv(2048).decode()
                if verbose:
                    print(x)
                time.sleep(wait_time)
        except Exception as e:
            print(e)
        finally:
            self.client.close()


if __name__ == '__main__':
    r1 = '192.168.200.101'
    username = 'admin'
    password = 'Cisc0123'
    client1 = QYTCiscoSSH(hostname=r1, username=username, password=password)
    # print(client1.show_run())
    cmds = ['show ver', 'show ip int br', '']
    # print(client1.show(cmds))
    client1.config(cmds)



import paramiko
from paramiko_expect import SSHClientInteraction


class QYTHuaweiSSH:
    def __init__(self, hostname, username, password):
        self.client = paramiko.SSHClient()
        self.client.load_system_host_keys()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(hostname=hostname, username=username, password=password)

    def dis_cur(self, verbose=False):
        try:
            with SSHClientInteraction(self.client, timeout=20, display=verbose) as interact:
                interact.expect('<[\S\s]+>')
                screen_change_cmds = ['system-view', 'user-interface vty 0 4', 'screen-length 0']
                for cmd in screen_change_cmds:
                    interact.send(cmd)
                    interact.expect('\[[\S\s]+\]')
                interact.send('return')
                interact.expect('<[\S\s]+>')

                interact.send('display current-configuration')
                interact.expect('return', timeout=5)
                cmd_output = interact.current_output_clean
                return cmd_output
        except Exception as e:
            print(e)
        finally:
            self.client.close()

    def display(self, display_cmds, verbose=False):
        try:
            return_result_list = []
            with SSHClientInteraction(self.client, timeout=20, display=verbose) as interact:
                interact.expect('<[\S\s]+>')
                screen_change_cmds = ['system-view', 'user-interface vty 0 4', 'screen-length 0']
                for cmd in screen_change_cmds:
                    interact.send(cmd)
                    interact.expect('\[[\S\s]+\]')
                interact.send('return')
                interact.expect('<[\S\s]+>')
                for cmd in display_cmds:
                    interact.send(cmd)
                    interact.expect('<[\S\s]+>')
                    cmd_output = interact.current_output_clean
                    return_result_list.append(cmd_output)
            return return_result_list
        except Exception as e:
            print(e)
        finally:
            self.client.close()

    def config(self, config_cmds, verbose=True):
        try:
            config_cmds.insert(0, 'system-view')
            with SSHClientInteraction(self.client, timeout=20, display=verbose) as interact:
                interact.expect('<[\S\s]+>')
                for cmd in config_cmds:
                    interact.send(cmd)
                    interact.expect('\[[\S\s]+\]')
        except Exception as e:
            print(e)
        finally:
            self.client.close()


if __name__ == '__main__':
    r1 = '192.168.1.151'
    r2 = '192.168.1.152'
    username = 'admin'
    password = 'Cisc0123'
    client1 = QYTHuaweiSSH(hostname=r1, username=username, password=password)
    client2 = QYTHuaweiSSH(hostname=r2, username=username, password=password)

    # 查看Current Config
    print(client1.dis_cur())

    # 配置OSPF
    c_cmds_1 = ['sysname AR1',
                'interface LoopBack 0',
                'ip address 1.1.1.1 32',
                'interface GigabitEthernet 0/0/2',
                'ip address 10.1.1.1 24',
                'ospf 1 router-id 1.1.1.1',
                'area 0.0.0.0',
                'network 10.1.1.0 0.0.0.255',
                'network 1.1.1.1 0.0.0.0'
                ]

    c_cmds_2 = ['sysname AR2',
                'interface LoopBack 0',
                'ip address 2.2.2.2 32',
                'interface GigabitEthernet 0/0/2',
                'ip address 10.1.1.2 24',
                'ospf 1 router-id 2.2.2.2',
                'area 0.0.0.0',
                'network 10.1.1.0 0.0.0.255',
                'network 2.2.2.2 0.0.0.0'
                ]
    client1.config(c_cmds_1, verbose=True)
    client2.config(c_cmds_2, verbose=True)

    snmp_cmds = ['snmp-agent sys-info version v2c',
                 'snmp-agent sys-info contact collinsctk',
                 'snmp-agent sys-info location beijing_qyt',
                 'snmp-agent community read QytangR0'
                 ]

    client1.config(snmp_cmds.copy(), verbose=True)
    client2.config(snmp_cmds.copy(), verbose=True)

    # 其他查看内容
    for r in client1.display(['display ospf peer', 'dis ver']):
        print(r)



from qyt_devnet.modules.snmp_v2_get import snmpv2_get
from qyt_devnet.modules.snmp_v2_getnext import snmpv2_getnext


class QYTHuaweiSNMP:
    def __init__(self, ip, ro, port=161):
        self.ip = ip
        self.ro = ro
        self.port = port

    def sys_desc(self):
        return snmpv2_get(self.ip, self.ro, "1.3.6.1.2.1.1.1.0", port=self.port)[1]

    def hostname(self):
        return snmpv2_get(self.ip, self.ro, "1.3.6.1.2.1.1.5.0", port=self.port)[1]

    def location(self):
        return snmpv2_get(self.ip, self.ro, "1.3.6.1.2.1.1.6.0", port=self.port)[1]

    def cpu_usage(self):
        return int(snmpv2_get(self.ip, self.ro, "1.3.6.1.4.1.2011.5.25.31.1.1.1.1.5.67108873", port=self.port)[1])

    def mem_usage(self):
        return int(snmpv2_get(self.ip, self.ro, "1.3.6.1.4.1.2011.5.25.31.1.1.1.1.7.67108873", port=self.port)[1])

    def get_ifs(self):
        return [if_name[1] for if_name in snmpv2_getnext(self.ip, self.ro, "1.3.6.1.2.1.2.2.1.2", port=161)]

    def get_if_speed(self):
        return [[name, speed] for name, speed in zip(self.get_ifs(), [if_speed[1] for if_speed in snmpv2_getnext(self.ip, self.ro, "1.3.6.1.2.1.2.2.1.5", port=161)])]

    def get_if_in_bytes(self):
        return [[name, speed] for name, speed in zip(self.get_ifs(), [if_speed[1] for if_speed in snmpv2_getnext(self.ip, self.ro, "1.3.6.1.2.1.2.2.1.10", port=161)])]

    def get_if_out_bytes(self):
        return [[name, speed] for name, speed in zip(self.get_ifs(), [if_speed[1] for if_speed in snmpv2_getnext(self.ip, self.ro, "1.3.6.1.2.1.2.2.1.16", port=161)])]


if __name__ == '__main__':
    r1 = '192.168.1.151'
    community_ro = "QytangR0"
    snmp_client = QYTHuaweiSNMP(r1, community_ro)
    # 系统描述
    print(snmp_client.sys_desc())
    # 主机名
    print(snmp_client.hostname())
    # 地点
    print(snmp_client.location())
    # CPU利用率
    print(snmp_client.cpu_usage())
    # 内存利用率
    print(snmp_client.mem_usage())
    # 接口清单
    print(snmp_client.get_ifs())
    # 接口速率
    print(snmp_client.get_if_speed())
    # 接口入向字节数
    print(snmp_client.get_if_in_bytes())
    # 接口出向字节数
    print(snmp_client.get_if_out_bytes())



#!/usr/bin/env python3
# -*- coding=utf-8 -*-
# 本脚由亁颐堂现任明教教主编写，用于乾颐盾Python课程！
# 教主QQ:605658506
# 亁颐堂官网www.qytang.com
# 教主技术进化论拓展你的技术新边疆
# https://ke.qq.com/course/271956?tuin=24199d8a


from pysnmp.hlapi import *


def snmpv2_get(ip, community, oid, port=161):
    # varBinds是列表，列表中的每个元素的类型是ObjectType（该类型的对象表示MIB variable）
    error_indication, error_status, error_index, var_binds = next(
        getCmd(SnmpEngine(),
               CommunityData(community),  # 配置community
               UdpTransportTarget((ip, port)),  # 配置目的地址和端口号
               ContextData(),
               ObjectType(ObjectIdentity(oid))  # 读取的OID
               )
    )
    # 错误处理
    if error_indication:
        print(error_indication)
    elif error_status:
        print('%s at %s' % (
            error_status,
            error_index and var_binds[int(error_index) - 1][0] or '?'
        )
              )
    # 如果返回结果有多行,需要拼接后返回
    result = ""

    for varBind in var_binds:

        result = result + varBind.prettyPrint() # 返回结果！
    # 返回的为一个元组,OID与字符串结果
    # print(result)
    return result.split("=")[0].strip(), result.split("=")[1].strip()


if __name__ == "__main__":
    # 使用Linux解释器 & WIN解释器
    # 系统描述
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.2.1.1.1.0", port=161))
    # 联系人
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.2.1.1.4.0", port=161))
    # 主机名
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.2.1.1.5.0", port=161))
    # 地点
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.2.1.1.6.0", port=161))
    # CPU利用率
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.4.1.2011.5.25.31.1.1.1.1.5.67108873", port=161))
    # 内存利用率
    print(snmpv2_get("192.168.1.151", "QytangR0", "1.3.6.1.4.1.2011.5.25.31.1.1.1.1.7.67108873", port=161))





#!/usr/bin/env python
# coding: utf-8

from setuptools import setup
import setuptools

setup(
    name='qyt_devnet',
    version='1.0.0',
    author='cq_bomb',
    author_email='collinsctk@qytang.com',
    url='https://github.com/collinsctk/DevNet_Huawei',
    license='GNU General Public License v3.0',
    description=u'乾颐堂网络设备自动化封装',
    packages=setuptools.find_packages(),
    # packages=['qyt_devnet'],
    install_requires=['paramiko', 'paramiko_expect', 'pysnmp'],
    python_requires='>=3.6',
)